/*
Copyright Rene Rivera 2011-2012
Distributed under the Boost Software License, Version 1.0.
(See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)
*/

#ifndef BOOST_PREDEF_DETAIL_TEST_H
#define BOOST_PREDEF_DETAIL_TEST_H

#if !defined(BOOST_PREDEF_INTERNAL_GENERATE_TESTS)

#define BOOST_PREDEF_DECLARE_TEST(x,s)

#endif

#endif
