#ifndef PYTHONIC_INCLUDE_OPERATOR_TRUEDIV__HPP
#define PYTHONIC_INCLUDE_OPERATOR_TRUEDIV__HPP

#include "pythonic/include/utils/functor.hpp"
#include "pythonic/include/operator_/truediv.hpp"

PYTHONIC_NS_BEGIN

namespace operator_
{
  USING_FUNCTOR(__truediv__, truediv)
}
PYTHONIC_NS_END

#endif
