#ifndef PYTHONIC_INCLUDE_NUMPY_FMAX_HPP
#define PYTHONIC_INCLUDE_NUMPY_FMAX_HPP

#include "pythonic/include/numpy/maximum.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{
  USING_FUNCTOR(fmax, maximum);
}
PYTHONIC_NS_END

#endif
