#ifndef PYTHONIC_OPERATOR_ILSHIFT_HPP
#define PYTHONIC_OPERATOR_ILSHIFT_HPP

#include "pythonic/include/operator_/ilshift.hpp"

#define OPERATOR_NAME ilshift
#define OPERATOR_SYMBOL <<
#define OPERATOR_ISYMBOL <<=

#include "pythonic/operator_/icommon.hpp"

#endif
