#ifndef PYTHONIC_INCLUDE_NUMPY_AMAX_HPP
#define PYTHONIC_INCLUDE_NUMPY_AMAX_HPP

#include "pythonic/include/numpy/max.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{

  USING_FUNCTOR(amax, max);
}
PYTHONIC_NS_END

#endif
