#ifndef PYTHONIC_INCLUDE_NUMPY_ARRAYSTR_HPP
#define PYTHONIC_INCLUDE_NUMPY_ARRAYSTR_HPP

#include "pythonic/include/numpy/array2string.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{
  USING_FUNCTOR(array_str, array2string);
}
PYTHONIC_NS_END

#endif
